package com.lzmispt.data.resource.service.datasync.impl.handle;

package .tmp;

import com.lzmispt.data.center.entity.dto.databasestruct.DBTableDTO;
import com.lzmispt.data.resource.entity.vo.datasync.DataSourceTableTreeVO;
import com.lzmispt.data.resource.service.datasync.impl.handle.AbstrcatDataSyncDatabase;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.dcconst.DCConst;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

@Service
public class PgsqlDataSyncDataBaseHandler extends AbstrcatDataSyncDatabase {
  public List<DataSourceTableTreeVO> getDataBaseList(Long dataSourceId, String platCode, String tokenId, Boolean isEnableCdc) {
    DataStructProvider dataStructProvider = getDataStructProvider(dataSourceId, platCode, tokenId, null, null);
    List<String> dataBaseNameList = (List<String>)this.dataStandardClient.getDataBaseNameList(dataStructProvider.currentDataSource, platCode, tokenId).getObj();
    Collections.sort(dataBaseNameList, (Comparator<? super String>)new AlphabetComparator());
    List<DataSourceTableTreeVO> dataSourceTableTreeList = (List<DataSourceTableTreeVO>)dataBaseNameList.stream().map(dataBaseName -> buildDataSourceTableVO(dataBaseName, null, null, dataSourceId, dataStructProvider.getDbType(), DCConst.DirectoryType.DATABASE)).collect(Collectors.toList());
    return dataSourceTableTreeList;
  }
  
  public List<DataSourceTableTreeVO> getSchemaList(DataStructProvider dataStructProvider, Long dataSourceId, String type, String dataBaseName, Boolean eanbleCdc) {
    dataStructProvider = getDataStructProvider(dataSourceId, "Collection", "0", dataBaseName, null);
    List<String> schemaNameList = dataStructProvider.getPgsqlSchemas(dataBaseName);
    return (List<DataSourceTableTreeVO>)schemaNameList.stream().map(schemaName -> buildDataSourceTableVO(dataBaseName, schemaName, null, dataSourceId, type, DCConst.DirectoryType.DATASCHEMA)).collect(Collectors.toList());
  }
  
  public List<DataSourceTableTreeVO> getTableList(DataStructProvider dataStructProvider, String schemaName, Long dataSourceId) {
    List<String> dbTableList = dataStructProvider.getPgsqlTable(schemaName);
    Collections.sort(dbTableList, (Comparator<? super String>)new AlphabetComparator());
    return (List<DataSourceTableTreeVO>)dbTableList.stream().map(tableName -> buildDataSourceTableVO(dataStructProvider.currentDataSource.getDbName(), dataStructProvider.currentDataSource.getDbSchema(), tableName, dataSourceId, dataStructProvider.getDbType(), DCConst.DirectoryType.DATATABLE))
      .collect(Collectors.toList());
  }
  
  public List<DataSourceTableTreeVO> getDataSourceTableTreeVO(Long dataSourceId, String platCode, String tokenId) throws Exception {
    List<DataSourceTableTreeVO> dataBaseList = getDataBaseList(dataSourceId, platCode, tokenId, Boolean.TRUE);
    return buildDataSourceTableTree(dataSourceId, dataBaseList, platCode);
  }
  
  private List<DataSourceTableTreeVO> buildDataSourceTableTree(Long dataSourceId, List<DataSourceTableTreeVO> dataBaseList, String platCode) {
    for (DataSourceTableTreeVO dataSourceTableTreeVO : dataBaseList) {
      DataStructProvider dataStructProvider = getDataStructProvider(dataSourceId, platCode, "0", dataSourceTableTreeVO.getSourceDatabaseName(), null);
      List<DataSourceTableTreeVO> dataSourceChild = getDataSourceChild(dataSourceId, dataSourceTableTreeVO, dataStructProvider, platCode);
      dataSourceTableTreeVO.setChildNodes(dataSourceChild);
    } 
    return dataBaseList;
  }
  
  private List<DataSourceTableTreeVO> getDataSourceChild(Long dataSourceId, DataSourceTableTreeVO dataSourceTableTree, DataStructProvider dataStructProvider, String platCode) {
    List<Map<String, String>> pgsqlTableAndSchemas = dataStructProvider.getPgsqlTableAndSchemas();
    Map<String, List<String>> tree = convertToTreeStructure(pgsqlTableAndSchemas);
    return buildSchema(tree, dataStructProvider.currentDataSource.getDbName(), dataSourceId, dataStructProvider.getDbType());
  }
  
  public List<DataSourceTableTreeVO> buildSchema(Map<String, List<String>> tree, String dataBaseName, Long dataSourceId, String type) {
    List<DataSourceTableTreeVO> dataSourceTableTreeVOList = new ArrayList<>();
    for (String schema : tree.keySet()) {
      DataSourceTableTreeVO dataSourceTableTreeVO = buildDataSourceTableVO(dataBaseName, schema, null, dataSourceId, type, DCConst.DirectoryType.DATASCHEMA);
      List<String> tables = tree.get(schema);
      if (CollectionUtils.isEmpty(tables))
        break; 
      Collections.sort(tables, (Comparator<? super String>)new AlphabetComparator());
      dataSourceTableTreeVO.setChildNodes(buildTable(tables, dataBaseName, schema, dataSourceId, type));
      dataSourceTableTreeVOList.add(dataSourceTableTreeVO);
    } 
    return dataSourceTableTreeVOList;
  }
  
  public List<DataSourceTableTreeVO> buildTable(List<String> tables, String dataBaseName, String schema, Long dataSourceId, String type) {
    List<DataSourceTableTreeVO> dataSourceTableTreeVOList = new ArrayList<>();
    for (String table : tables) {
      if (!StringUtils.isEmpty(table)) {
        DBTableDTO dbTableDTO = new DBTableDTO();
        dbTableDTO.setTableName(table);
        dbTableDTO.setSchemaName(schema);
        DataSourceTableTreeVO dataSourceTableTreeVO = buildDataSourceTableVO(dataBaseName, schema, dbTableDTO.getTableName(), dataSourceId, type, DCConst.DirectoryType.DATATABLE);
        dataSourceTableTreeVOList.add(dataSourceTableTreeVO);
      } 
    } 
    return dataSourceTableTreeVOList;
  }
  
  public static Map<String, List<String>> convertToTreeStructure(List<Map<String, String>> data) {
    Map<String, List<String>> tree = new HashMap<>();
    for (Map<String, String> entry : data) {
      String schema = entry.get("table_schema");
      String table = entry.get("table_name");
      if (!tree.containsKey(schema))
        tree.put(schema, new ArrayList<>()); 
      ((List<String>)tree.get(schema)).add(table);
    } 
    return tree;
  }
}
